/*jshint esversion:6*/
var Cus01600_DesvioDeConsumoController = function(ScreenService, General, GeneralServices, templateManager, Cus01600_DesvioDeConsumoService) {
	/*global moment, Configuration, getLocalVar*/
	
	var self = this;

	/**
	 * Array de valores padrões a serem usados 
	 * em toda a tela.
	*/

	let defaultValues = {
		reportName : "cus01600",
		groupProdByLevel : "0",
		period : {
			start: moment().format('DD/MM/YYYY'),
			end: moment().format('DD/MM/YYYY'),
			formattedStart: moment().format('DD/MM/YYYY'),
			formattedEnd: moment().format('DD/MM/YYYY')
		},
		CDGRUPFILI : 'T',
		allGrupServ: 'Y',
		allCdServfili: 'Y'
	};
	
	var reportFormats = [
	    {
	        name: "pdf",
	        value: "PDF"
	    },
	    {
	        name: "xlsx",
	        value: "XLSX"
	    }
	];
	
	/**
	 * Função responsável por preencher os valores padrões
	 * na currentRow da widget passada por parâmetro.
	 * 
	 * @param {Widget} widget Widget a ser preenchida.
	*/
	
	this.applyDefaultValues = function(widget){
		Object.keys(defaultValues).forEach(key=>{
			widget.currentRow[key] = defaultValues[key];
		});
		self.agrupNivel(widget);
		templateManager.updateTemplate();
	};

	/**
	 * Conjunto de instruções a serem executadas ao alterar
	 * os campos checkbox 'Todos'
	 * 
	 * @param {Field} field Field checkbox 'Todos'
	*/
	
	this.onChangeCheckboxAll = function(widget, field){
		if(field.name === 'allCdFilial' || field.name === 'allGrupServ'){
			this.onChangeFilialGrupServ(field, widget);
		}
		
		let relatedField = field.widget.getField(field.relatedSelectField);
		General.toggleReadOnlySelect(field, relatedField);
	};
	
	this.onChangeFilialGrupServ = function(field, widget){
		var cdservfili = widget.getField('CDSERVFILI');
		let cRow = widget.currentRow;
		var listaCdFilial = cRow.allCdFilial == 'Y' ? 'T' : cRow.CDFILIAL;
		var nameField = field.name;
		cdservfili.clearValue();
		if(nameField === 'CDFILIAL' || nameField === 'allCdFilial'){
			insertDelLista(widget, cRow, listaCdFilial, field);
		}
	};
	
	function insertDelLista(widget, cRow, listaCdFilial, field){
		if(listaCdFilial != ''){
			if(widget.nrLista === true){
				Cus01600_DesvioDeConsumoService.deleteLista(widget.currentRow.NRLISTA).then(function(success) {
					widget.nrLista = false;
					insertLista(widget, cRow, listaCdFilial);
				}).catch(function(e){
			        ScreenService.showMessage(e);
			    });
			}else{
				insertLista(widget, cRow, listaCdFilial);
			}
		}else if((listaCdFilial === '' && field.name === 'agrupFili') || listaCdFilial.length === 0){
			Cus01600_DesvioDeConsumoService.deleteLista(widget.currentRow.NRLISTA).then(function(success) {
				widget.nrLista = false;
				cRow.NRLISTA = '';
			}).catch(function(e){
		        ScreenService.showMessage(e);
		    });
		}
	}
	
	function insertLista(widget, cRow, listaCdFilial){
		Cus01600_DesvioDeConsumoService.insertLista(listaCdFilial, cRow.CDGRUPFILI).then(function(success) {
			cRow.NRLISTA = success.dataset.data.NRLISTA;
			cRow.LISTACDFILIAL = success.dataset.data.LISTACDFILIAL;
			widget.nrLista = true;
		}).catch(function(e){
	        ScreenService.showMessage(e);
	    });
	}
	
	/**
	 * Conjunto de instruções a serem executadas ao entrar
	 * na widget de Desvio de Consumo.
	 * 
	 * @param {Widget} widget Widget Desvio de consumo.
	*/
	
	this.onEnterDesvioConsumo = function(widget){
		widget.nrLista = false;
		this.applyDefaultValues(widget);
	};

	/**
	 * Conjunto de insturções a serem executadas ao
	 * alterar o valor do campo de relatório.
	 * 
	 * @param {Widget} widget Widget Desvio de Consumo.
	*/
	
	this.onChangeReport = function(widget){
		this.setTipoPratStateByReportType(widget);
	};
	
	/**
	 * Define o estado dos fields relativos a Tipo 
	 * de Prato de acordo com o relatório escolhido.
	 * 
	 * @param {Widget} widget Widget Desvio de Consumo.
	*/
	
	this.setTipoPratStateByReportType = function(widget){
		let cRow 				= widget.currentRow;
		let cdTipoPratField 	= widget.getField("CDTIPOPRAT");
		let allCdTipoPratField 	= widget.getField("allCdTipoPrat");
		let condition 			= cRow.reportName!="cus01605";

		allCdTipoPratField.readOnly = condition;
		allCdTipoPratField.value(condition ? 'N' : 'S');
		
		if(cRow.reportName!="cus01605"){
			cdTipoPratField.clearValue();
		}

		cdTipoPratField.readOnly 	= condition;
		this.setRequiredField(cdTipoPratField, !condition);
	};

	/**
	 * Função responsável por definir validação required de campo passado por
	 * parâmetro.
	 * 
	 * @param {Field} field Campo a ser alterado.
	 * @param {Boolean} state Estado da validação required.
	*/
	
   	this.setRequiredField = function(field, state){
		if(state){
			
			if(field.validations){
				field.validations.required = true;
			} else {
				field.validations = {
					required : true
				};
			}
			
		} else {
			if(field.validations)
				delete field.validations.required;
		}
	};

	/**
	 * Função responsável por validar e limpar o campo
	 * de Agrupamento de Prodrutos por nível. 
	 * 
	 * @param {Args} args Argumentos do evento disparado
	 * no campo.
	*/

	this.agrupNivel = function(widget){
		let level = General.getLevelByMask('CDPICTPROD', 1);
		widget.currentRow.NRTAMCODIGO = level[level.length-1].caracteres;
		widget.currentRow.NMTAMACNIVEL = "0";
		if(level.length >= 2){
			level.pop();
			level.pop();
		}
		level.map(function(item){
			item.NMTAMACNIVEL = item.nivel;
		});
		level.unshift(
			{
				NMTAMACNIVEL: "0"
			}
		);
		widget.getField('groupProdByLevel').dataSource.data = level; 
	};

	/**
	 * Conjunto de instruções a serem executadas em caso
	 * de alteração do valor do campo de Agrupamento
	 * de Filiais.
	 * 
	 * @param {Field} field Field de Agrupamento
	 * 	de filiais.
	*/
	 
	this.onChangeAgrupFili = function(field){
		var cdfilial = field.widget.getField('CDFILIAL');
		var cdservfili = field.widget.getField('CDSERVFILI');
		var allCdFilial = field.widget.getField('allCdFilial');
		var listaCdFilial;
		if(field.value()==''){
			field.widget.currentRow.CDGRUPFILI = 'T';
			field.widget.currentRow.allCdFilial = "Y";
			allCdFilial.checked = true;
			this.onChangeCheckboxAll(field.widget, allCdFilial);
			listaCdFilial= 'T';
			templateManager.updateTemplate();
			insertDelLista(field.widget, field.widget.currentRow, listaCdFilial, field);
		}else{
			field.widget.currentRow.allCdFilial = 'Y';
			allCdFilial.checked = true;
			this.onChangeCheckboxAll(field.widget, allCdFilial);
			templateManager.updateTemplate();
			listaCdFilial = 'T';
			insertDelLista(field.widget, field.widget.currentRow, listaCdFilial, field);
		}
		cdservfili.clearValue();
		
	};
	
	/**
	 * Função responsável por definir filtro do campo
	 * de serviço.
	 * 
	 * @param {Field} field Field de Serviço.
	*/ 
	 
	this.setServicoFilter = function(field){
		
		let cRow = field.widget.currentRow;
		var idutretiprat = cRow.reportName == 'cus01604' ? 'S' : 'T';

		let nrlistafilter = field.dataSourceFilter.filter(filter=>filter.name=='NRLISTA')[0];
		let cdgrupservfilter = field.dataSourceFilter.filter(filter=>filter.name=='CDGRUPSERV')[0];
		let idutretipratfilter = field.dataSourceFilter.filter(filter=>filter.name=='IDUTRETIPRAT')[0];
	
		nrlistafilter.value = cRow.NRLISTA;
		if(cRow.allGrupServ == 'Y'){
			cdgrupservfilter.value = 'T';
		}else if(cRow.CDGRUPSERV && cRow.CDGRUPSERV.length){
			cdgrupservfilter.value = cRow.CDGRUPSERV.join('_');
		}else{
			cdgrupservfilter.value = '';
		}
		idutretipratfilter.value = idutretiprat;
		field.reload();
	};
	
	

	/**
	 * Função responsável por validar os dados
	 * e abrir a widget de relatório.
	 * 
	 * @param {Widget} widget Widget desvio de
	 * 	consumo.
	*/
	 	/* global angular */
	this.openReport = function(widget){	
		
		if(widget.isValid() && this.validaNivel(widget.currentRow) && this.validaTipoProduto(widget)){
			this.ordenaProduto(widget);
		    let cRow = angular.copy(widget.currentRow);
			if(cRow.allCdServfili != 'Y'){
				var index = 0;
				var listaFiliServ = [];
				var filiConcatServ;
				cRow.FILISERV.forEach(function (filial) {
					filiConcatServ = filial + cRow.CDSERVFILI[index];
					listaFiliServ.push(filiConcatServ);
					index++;
				});
				cRow.LISTAFILISERV = listaFiliServ;
			}else{
				cRow.LISTAFILISERV = 'T';
			}
			if(cRow.allCdTipoPrat =='N' && cRow.CDTIPOPRAT == undefined){
				cRow.CDTIPOPRAT = "X";	
			}
			let reportRow = {
				DTINICIAL 		 : cRow.period.start,
				DTFINAL 		 : cRow.period.end,
				IDESTOCAVEIS	 : cRow.estoc,
				IDNESTOCAVEIS	 : cRow.naoEstoc,
				reportName 		 : cRow.reportName,
				CDTIPOPRAT 		 : cRow.allCdTipoPrat =='Y' ? 'X' : cRow.CDTIPOPRAT,
				TCDTIPOPRAT		 : cRow.allCdTipoPrat =='Y' ? 'T' : 'X',
				CDGRUPFILI 		 : cRow.CDGRUPFILI,
				NRTAMCODIGO   	 : cRow.NRTAMCODIGO,
				NMCUSTO 		 : widget.getField('NMTIPOCUSTO').value().substring(5),
				NRTIPOCUSTO 	 : cRow.CDTIPOCUSTO,
				CDPRODINIC 	     : cRow.CDARVPRODINI,
				CDPRODFINA 	     : cRow.CDARVPRODFIN,
				NRTAMACNIVEL	 : cRow.groupProdByLevel,
				CDFILIAL		 : cRow.allCdFilial == 'Y' ? 'T' : cRow.CDFILIAL,
				CDGRUPSERV		 : cRow.allGrupServ == 'Y' ? 'T' : cRow.CDGRUPSERV,
				CDSERVFILI		 : cRow.allCdServfili == 'Y' ? 'T' : cRow.CDSERVFILI,
				IDDIFERENCA		 : cRow.diffBy,
				LISTAFILISERV    : cRow.LISTAFILISERV,
				NRLISTA          : cRow.NRLISTA,
				CDPICTPROD       : getLocalVar("CDPICTPROD").replace(/[^(/\.|\ )]/g, "@"),
				LISTACDFILIAL    : cRow.LISTACDFILIAL,
				DTMESINIC : moment(cRow.period.start, 'DD/MM/YYYY').startOf('month').format('DD/MM/YYYY'),
				DTMESFINA : moment(cRow.period.end, 'DD/MM/YYYY').endOf('month').format('DD/MM/YYYY'),
				IDQUEBRAPAGINA	 : cRow.QUEBRAPAG
			};
	
			General.openReportPopup(widget, this.generateReport, [widget.currentRow, reportRow], reportFormats);
		}
	};

	this.validaTipoProduto = function(widget){
		if(widget.currentRow.estoc == 'N' && widget.currentRow.naoEstoc == 'N'){
			ScreenService.showMessage('O tipo de produto não foi informado', 'error');
			return false;
		}else{
			return true;
		}
	};

	/**
	 * Função responsável por gerar o relatório 
	 * passando os dados de forma correta.
	 * 
	 * @param {Row} widgetRow CurrentRow da widget de
	 * 	desvio de consumo.
	 * @param {Row} reportRow Row criada somente com os
	 * 	dados do relatório.
	*/
	 
	this.generateReport = function(widgetRow, reportRow) {
		reportRow.QR2TYPE = widgetRow.QR2TYPE;
		General.openReport('cus_relatorio#/cus01600_desvio_de_consumo', reportRow.reportName, reportRow);
	};
	
	/**
	 * Função responsável setar o valor do campo
	 * NRTAMCODIGO na forma correta.
	 * 
	 * @param {Widget} widget Widget desvio de
	 * 	consumo.
	*/
	
	this.ordenaProduto = function(widget) {
		var ordenacao_tipo = widget.getField('reportOrder').value();
		if(ordenacao_tipo == "0") {
		 	var tamanho_mascara = this.concatenaArrayMascara(getLocalVar('CDPICTPROD').split('.'));
		 	widget.getField('NRTAMCODIGO').setValue(tamanho_mascara);
		} else if(ordenacao_tipo == "1") {
			widget.getField('NRTAMCODIGO').setValue(0);
		}
	};
	
	/**
	 * Função responsável por retornar o valor
	 * para ser setado o NRTAMCODIGO.
	 * 
	 * @param {Array} array Array valor da
	 * local var CDPICTPROD.
	 * 
	*/
	 
	this.concatenaArrayMascara = function(array) {
		arrayString = '';
		for(var pos in array) {
			arrayString += array[pos];
		}
		return arrayString.length;
	};
	
	this.validaNivel = function (currentRow) {
		var mensagem = "O produto final deve ser maior que o inicial.";
		var retorno = '';

		if (currentRow.CDARVPRODINI && currentRow.CDARVPRODFIN) {
			retorno = General.comparaPratoProduto(currentRow.CDARVPRODINI, currentRow.CDARVPRODFIN, getLocalVar('CDPICTPROD'));
			if (!retorno) {
				ScreenService.showMessage(mensagem);
				return false;
			}
			return true;
		}
		return false;
	};
	
	this.alteraProdutoFinal = function(widget){
	  widget.currentRow.prodFin = angular.copy(widget.currentRow.prodIni);
	  widget.currentRow.CDPRODUTOFIN = angular.copy(widget.currentRow.CDPRODUTOINI);
	  widget.currentRow.CDARVPRODFIN = angular.copy(widget.currentRow.CDARVPRODINI);
	};
	
};

Configuration(function(ContextRegister) {
	ContextRegister.register('Cus01600_DesvioDeConsumoController', Cus01600_DesvioDeConsumoController);
});